package fi.helsinki.w3m;

import fi.helsinki.w3m.filter.*;
import fi.helsinki.w3m.metadata.MuseumModel;
import java.net.URL;
import java.net.MalformedURLException;
import junit.framework.Assert;

public class W3m {
    static MuseumModel model=null;
    static Filter filter=null;
    
    public static Filter getFilter() {
	return getFilter("../../../../data/");
    }
    
    public static MuseumModel getModel() {
	return getModel("../../../../data/");
    }

    public static MuseumModel getModel(String configfilename) {
	if (model==null) {
	    try {
		URL url=new URL("http://www.cs.helsinki.fi/group/wwwmuseo/kuvat/");
		model=new MuseumModel(url,configfilename);
		if (!model.readModel()) Assert.fail("Could not read model from file");
	    } catch (MalformedURLException e) {
		Assert.fail("Malformed URL while constructing MuseumModel");
	    }
	}
	return model;
    }

    public static Filter getFilter(String configfilename) {
	if (filter==null) {
	    getModel(configfilename);
	    
	    UtilityFunctionChain tmp;
	    // initialize utility functions for links
	    tmp=new UtilityFunctionChain();
	    
	    // random utilityfunction
	    tmp.addUtilityFunction(new RandomThresholdUtilityFunction(),10);
	    // how well the target image of the link corresponds to the current facetselection
	    tmp.addUtilityFunction(new InterestingLinkFacetUtilityFunction(model),1);
	    // how popular is the link with respect to the whole data
	    tmp.addUtilityFunction(new InterestingLinkGlobalAccessCountUtilityFunction(model),1);
	    // how popular is the link with respect to the links in the image
	    tmp.addUtilityFunction(new InterestingLinkImageLinkAccessCountUtilityFunction(),1);
	    // how popular is the target image of the link with respect to the whole data
	    tmp.addUtilityFunction(new InterestingLinkTargetImageGlobalAccessCountUtilityFunction(model),1);
	    // how popular is the target image of the link with respect to the links in the image
	    tmp.addUtilityFunction(new InterestingLinkTargetImageLocalAccessCountUtilityFunction(),1);
	    tmp.addUtilityFunction(new PreviousImageUtilityFunction(model),10);
	    UtilityFunction interestingLinkUtilityFunction=tmp;
	    
	    // initialize utility functions for images
	    tmp=new UtilityFunctionChain();
	    // random utilityfunction
	    tmp.addUtilityFunction(new RandomThresholdUtilityFunction(),1);
	    // how popular is the image with respect to the whole data
	    tmp.addUtilityFunction(new InterestingImageAccessCountUtilityFunction(model),1);
	    tmp.addUtilityFunction(new PreviousImageUtilityFunction(model),10);
	    UtilityFunction interestingImageUtilityFunction=tmp;
	    
	    // initialize utility functions for ontology classes
	    tmp=new UtilityFunctionChain();
	    // how popular is the image with respect to the whole data
	    tmp.addUtilityFunction(new InterestingOntologyClassAccessCountUtilityFunction(model),1);
	    UtilityFunction interestingOntologyClassUtilityFunction=tmp;
	    
	    filter=new Filter(getModel(configfilename),interestingLinkUtilityFunction, interestingImageUtilityFunction, interestingOntologyClassUtilityFunction);
	}
	return filter;
    }
}
