package fi.helsinki.w3m.filter;
import fi.helsinki.w3m.metadata.item.*;
import fi.helsinki.w3m.metadata.ontology.*;
import fi.helsinki.w3m.metadata.*;
import fi.helsinki.w3m.profile.Profile;
import java.util.*;

/*
 * $Author: jiemakel $
 * $Revision: 1.65 $
 * $Date: 2002/05/08 12:19:26 $
 */

public class Filter {
    
    private MuseumModel model;
    private UtilityFunction interestingLinkUtilityFunction;
    private UtilityFunction interestingImageUtilityFunction;
    private UtilityFunction interestingOntologyClassUtilityFunction;

    public static final int DIRECT=0;
    public static final int ALL=-1;
    
    public Filter(MuseumModel model) {
	this.model=model;
    }

    public Filter(MuseumModel model, UtilityFunction ilu, UtilityFunction iiu, UtilityFunction iocu) {
	this.model=model;
	this.interestingLinkUtilityFunction=ilu;
	this.interestingImageUtilityFunction=iiu;
	this.interestingOntologyClassUtilityFunction=iocu;
    }

    public boolean setInterestingLinkUtilityFunction(UtilityFunction uf) {
	interestingLinkUtilityFunction=uf;
	return true;
    }

    public UtilityFunction getInterestingLinkUtilityFunction() {
	return interestingLinkUtilityFunction;
    }

    public boolean setInterestingImageUtilityFunction(UtilityFunction uf) {
	interestingImageUtilityFunction=uf;
	return true;
    }

    public UtilityFunction getInterestingImageUtilityFunction() {
	return interestingImageUtilityFunction;
    }

    public boolean setInterestingOntologyClassUtilityFunction(UtilityFunction uf) {
	interestingOntologyClassUtilityFunction=uf;
	return true;
    }

    public UtilityFunction getInterestingOntologyClassUtilityFunction() {
	return interestingOntologyClassUtilityFunction;
    }

/*
*This method sorts the given InterestingLink objects by 
*interestingLinkUtilityFunction of the caller. It returns only max or less 
*number of links.
*
*@param interestingLinks These are the InterestingLink objects to 
*be sorted.
*@param ui The UtilityInformation of the caller
*@param max The integer of maximum number of InterestingLink objects to be 
*returned
*@returns interestingLinks This is the vector of sorted InterestingLink 
*objects.
*/


    public Collection filterInterestingLinks(Vector interestingLinks, UtilityInformation ui, int max) {
	/*
	 * Save the utilities in the objects
	 */
	for (Iterator i=interestingLinks.iterator(); i.hasNext();) {
	    InterestingLink il=(InterestingLink)i.next();
	    il.calculateUtility(interestingLinkUtilityFunction,ui);
	}
	/*
	 * Sort the Vector. The utilities HAVE TO HAVE BEEN SAVED IN THE OBJECTS FOR THIS TO WORK.
	 */
	Collections.sort(interestingLinks);
	Collections.reverse(interestingLinks);
	if (max>0 && interestingLinks.size()>max) interestingLinks.setSize(max); // cut the Vector if it is longer than max 
	return interestingLinks;	
    }
    
    /** Find links for an image and return them sorted by interestingness
     * @param imageID ID of the image we are getting the links for
     * @param profile Profile of the calling user
     * @param max Maximum number of links returned
     * @returns a Vector of maximum max InterestingLinks
     */
    public Collection getInterestingLinks(Id imageID, Profile profile, int max) {	
	//initialization
	Image image=model.getImage(imageID);
	
	/*
	 * Iterate over the properties and the images we get through 
	 * them to populate the InterestingLink Vector
	 */

	ResourceCollection properties=image.getProperties();
	Vector interestingLinks=new Vector();
	for (Iterator i=properties.iterator(); i.hasNext();) {
	    ResourceProperty rp=(ResourceProperty)i.next();
	    PropertyValue pv=rp.getValue();
	    if (pv.isInstance()) {
		ResourceCollection images=((Instance)pv.getValue()).getImages();
		for (Iterator j=images.iterator(); j.hasNext();) {
		    Image im=(Image)j.next();
		    if (!im.equals(image)) {
			InterestingLink il=new InterestingLink(rp,im);
			int iof = interestingLinks.indexOf(il);
			if (iof==-1) interestingLinks.add(il);
			else {
			    il = (InterestingLink)interestingLinks.get(iof);
			    il.addProperty(rp);
			    il.setLinkName(il.getLinkName()+", "+rp.getLabel());
			    if (rp.getValue().getInstance().getLabel()!=null) 
				il.setLinkName(il.getLinkName()+": "+rp.getValue().getInstance().getLabel());
			}
		    }
		}
		
	    }
	}
	
	/*	System.out.print("Got ");
		System.out.print(interestingLinks.size());
		System.out.print(" links for image ");
		System.out.print(imageID);
		System.out.print(" through ");
		System.out.print(properties.size());
		System.out.println(" properties."); */
	
	UtilityInformation ui=new UtilityInformation(image,profile,interestingLinks);
	return filterInterestingLinks(interestingLinks,ui,max);
    }	
    
    /** Find links for an image and return them sorted by interestingness
     * @param imageID ID of the image we are getting the links for
     * @param profile Profile of the calling user
     * @returns a Vector of InterestingLinks
     */
    public Collection getInterestingLinks(Id imageID,Profile profile) {
	return getInterestingLinks(imageID,profile,ALL);
    }

    public Collection filterInterestingOntologyClasses(Vector interestingOntologyClasses,UtilityInformation ui,int max) {
	/*
	 * Save the utilities in the objects
	 */
	for (Iterator i=interestingOntologyClasses.iterator(); i.hasNext();) {
	    InterestingOntologyClass ioc=(InterestingOntologyClass)i.next();
	    ioc.calculateUtility(interestingOntologyClassUtilityFunction,ui);
	}

	/*
	 * Sort the Vector. The utilities HAVE TO HAVE BEEN SAVED IN THE OBJECTS FOR THIS TO WORK.
	 */
	
	Collections.sort(interestingOntologyClasses);
	Collections.reverse(interestingOntologyClasses);
	if (max>0 && interestingOntologyClasses.size()>max) interestingOntologyClasses.setSize(max); // cut the Vector if it is longer than max 
	return interestingOntologyClasses;
    }

    public Collection filterOntologyClasses(Collection ontologyClasses,UtilityInformation ui, int max) {
        Vector interestingOntologyClasses = new Vector();

	for (Iterator i=ontologyClasses.iterator(); i.hasNext();) {
	    OntologyClass oc=(OntologyClass)i.next();
	    interestingOntologyClasses.add(new InterestingOntologyClass(oc));
	}

	return filterInterestingOntologyClasses(interestingOntologyClasses,ui,max);
    }
    
    /** Get facet roots sorted by interestingness
     * @param profile Profile of the calling user
     * @param max Maximum number of facets returned
     * @returns A maximum of max facet roots as a Vector of OntologyClass objects.
     */
    public Collection getFacetRoots(Profile profile, int max) throws com.hp.hpl.mesa.rdf.jena.model.RDFException {
	Collection ontologyClasses =  model.getFacetRoots();
	UtilityInformation ui=new UtilityInformation(profile,ontologyClasses);
	return filterOntologyClasses(ontologyClasses,ui,max);
    }

    /** Get facet roots sorted by interestingness
     * @param profile Profile of the calling user
     * @returns The facet roots as a Vector of OntologyClass objects.
     */
    public Collection getFacetRoots(Profile profile) throws com.hp.hpl.mesa.rdf.jena.model.RDFException {
	return getFacetRoots(profile,ALL);
    }
    
    /** Get children for a facet node
     * @param node 
     * @param profile Profile of the calling user
     * @returns The children of the facet node as a Vector of OntologyClass objects.
     */
    public Collection getFacetChildren(OntologyClass node, Profile profile) throws com.hp.hpl.mesa.rdf.jena.model.RDFException {
	Collection children = node.getChildren(0);
	UtilityInformation ui=new UtilityInformation(profile,children);
	return filterOntologyClasses(children,ui,ALL);
    }
    
    public Collection filterInterestingImages(Vector interestingImages,UtilityInformation ui,int max) {	
	/*
	 * Save the utilities in the objects
	 */
	for (Iterator i=interestingImages.iterator(); i.hasNext();) {
	    InterestingImage ii=(InterestingImage)i.next();
	    ii.calculateUtility(interestingImageUtilityFunction,ui);
	}

	/*
	 * Sort the Vector. The utilities HAVE TO HAVE BEEN SAVED IN THE OBJECTS FOR THIS TO WORK.
	 */
	
	Collections.sort(interestingImages);
	Collections.reverse(interestingImages);

	return interestingImages;	
    }

    public Collection filterImages(Collection images,UtilityInformation ui, int max) {
	Vector interestingImages=new Vector();

	for (Iterator i=images.iterator(); i.hasNext();) {
	    Image im=(Image)i.next();
	    interestingImages.add(new InterestingImage(im));
	}

	return filterInterestingImages(interestingImages,ui,max);
    }

    /** Get default images to show to the user
     * @param profile Profile of the calling user
     * @param max maximum number of images to return
     * @returns at most max most popular Images
     */
    public Collection getDefaultImages(Profile profile, int max) {
	Collection images=model.getPopularImages(max);
	UtilityInformation ui=new UtilityInformation(profile,images);
	return filterImages(images,ui,max);
    }

    /** Get images that are contained in an intersection of HiBrowse selections
     * @param ontologyClass the node whose children we are filtering
     * @param profile Profile of the calling user
     * @returns a Vector of InterestingImages in the intersection
     */
    public Collection getIntersectImages(OntologyClass ontologyClass, Profile profile) {
	return getIntersectImages(ontologyClass,profile,ALL);
    }
    
    /** Get images that are contained in an intersection of HiBrowse selections
     * @param ontologyClass the node whose children we are filtering
     * @param profile Profile of the calling user
     * @param max maximum depth to get intersections from
     * @returns a Vector of InterestingImages in the intersection
     */
    public Collection getIntersectImages(OntologyClass ontologyClass, Profile profile, int max) {
	Collection images;
	//	if (max>0) images=ontologyClass.intersect((Collection)profile.getStateOfHiBrowse(),max); else 
	images=ontologyClass.intersect(profile.getStateOfHiBrowse());
	UtilityInformation ui=new UtilityInformation(profile,images);
	return filterImages(images,ui,max);
    }

    public boolean setModel(MuseumModel model) {
	this.model=model;
	return true;
    }
    
    public MuseumModel getModel() {
      return model;
    }
}
