package fi.helsinki.w3m.filter;
import fi.helsinki.w3m.profile.Profile;
import fi.helsinki.w3m.metadata.item.Image;
import fi.helsinki.w3m.metadata.ontology.*;
import fi.helsinki.w3m.metadata.MuseumModel;
import java.util.Vector;

/*
 * $Author: jiemakel $
 * $Revision: 1.31 $
 * $Date: 2002/05/10 12:42:46 $
 */

public class InterestingLink implements Comparable {

    private int utility;
    private Image image;
    private Vector properties=new Vector();
    private String linkname;

    //constructors
    
    public InterestingLink(String name){
	this.linkname=name;
    }
    
    public InterestingLink(Image im){
	this.image=im;
    }
    
    public InterestingLink(int util){
	setUtility(util);
    }
    
    public InterestingLink(String name, Image im, int util){
	this.linkname=name;
	this.image=im;
	setUtility(util);
    }
    
    public InterestingLink(String name, Image im){
	this.linkname=linkname;
	this.image=im;
    }
    
    public InterestingLink(String name, int util){
	this.linkname=name;
	setUtility(util);
    }
    
    public InterestingLink(Image im, int util){
	this.image=im;
	this.linkname=MuseumModel.decodeURI(im.getLabel());
	setUtility(util);
    }
    
    public InterestingLink(ResourceProperty ip, Image im) {
	if (im.getSubject()!=null) {
	    this.linkname=im.getSubject();
	} else {
	    int lastindex=50;
	    String description=im.getDescription();
	    if (description.length()<lastindex) lastindex=description.length();
	    else if (description.indexOf(". ",lastindex)!=-1) {
		lastindex=lastindex=description.indexOf(". ",lastindex)+1;
		while (description.indexOf(". ",lastindex)!=-1 && description.indexOf(".",lastindex-10)!=description.indexOf(". ",lastindex)) lastindex=description.indexOf(". ",lastindex)+1;
	    }
		else lastindex=description.length();
	    if (lastindex>80) {
		lastindex=80;
		if (description.lastIndexOf(",",lastindex)!=-1)
		    lastindex=description.lastIndexOf(",",lastindex);
		else if (description.lastIndexOf(" ",lastindex)!=-1)
		    lastindex=description.lastIndexOf(" ",lastindex);
		this.linkname=description.substring(0,lastindex)+"...";
	    } else 
		this.linkname=description.substring(0,lastindex);
	}
	this.linkname+="<br>"+ip.getLabel();
	if (ip.getValue().getInstance().getLabel()!=null) this.linkname+=": "+ip.getValue().getInstance().getLabel();
	properties.add(ip);
	this.image=im;
    }
    
    //return an Image of InterestingLink
    public Image getImage(){
	return image;
    }

    //set a received Image to an InterestingLink
    
    public boolean setImage(Image im ){
	this.image=im;
	return true;
    }

    //return the Property of InterestingLink
    public Vector getProperties(){
	return properties;
    }

    //set a received Property to an InterestingLink
    
    public boolean addProperty(ResourceProperty ip ){
	properties.add(ip);
	return true;
    }

    public boolean setProperties(Vector props) {
	properties=props;
	return true;
    }
    
    public String getLinkName(){
	return linkname;
    }

    public boolean setLinkName(String name){
	this.linkname=name;
	return true;
    }

    public boolean setUtility(int utility) {
	this.utility=utility;
	return true;
    }

    public int getUtility() {
	return utility;
    }

    public int calculateUtility(UtilityFunction uf, UtilityInformation ui) {
	setUtility(uf.getUtility(this,ui));
	return getUtility();
    }

    public int compareTo(Object o) {
	InterestingLink il=(InterestingLink)o;
	return getUtility()-il.getUtility();
    }

    public boolean equals(Object o) {
	InterestingLink oil = (InterestingLink)o;
	if (this.getImage().equals(oil.getImage())) {
	    return true;
	} else {
	    return false;
	}
    }

}
