package fi.helsinki.w3m.filter;
import fi.helsinki.w3m.profile.*;
import fi.helsinki.w3m.metadata.*;
import fi.helsinki.w3m.metadata.item.Image;
import fi.helsinki.w3m.metadata.ontology.*;
import java.util.*;


/*
 * $Author: jiemakel $
 * $Revision: 1.17 $
 * $Date: 2002/05/10 10:43:25 $
 */

public class InterestingLinkFacetUtilityFunction extends UtilityFunction {
    private MuseumModel model;
    
    public InterestingLinkFacetUtilityFunction(MuseumModel model){
	this.model=model;
    }

    private class MyFacetCombination {
	OntologyClass facetRoot;
	OntologyClass facetNode;
	int nodeDepth;
	MyFacetCombination (OntologyClass facetNode) {
	    this.facetNode=facetNode;
	    try {
		Collection facetroots = model.getFacetRoots();
		OntologyClass o=facetNode;
		int i=0;
		while (!facetroots.contains(o)) {
		    Collection parents=o.getParents(OntologyClass.DIRECT);
		    Iterator itr=parents.iterator();
		    if (itr.hasNext()) {
			o=(OntologyClass)itr.next();
		    }
		    i++;
		}
		facetRoot=o;
		nodeDepth=i;
	    } catch (Exception e) {	    
	    }
	}

	public boolean equals(Object o) {
	    MyFacetCombination ofs=(MyFacetCombination)o;
	    if (facetRoot!=null && ofs.facetRoot!=null) {
		return facetRoot.equals(ofs.facetRoot);
	    } else return false;
	}
    }
    

    /**
     *
     * This method calculates the utility of a an image of a link to
     * the the current state of HiBrowse. This means getting an inverse
     * of the distance of the image the current state of HiBrowse.
     * @param il This is received InterestingLink
     * @param profile The Profile of the caller
     * @returns an integer as an utility 
     */
    
    public int getUtility(InterestingLink il, UtilityInformation ui){
	int path=0;
	Image image=il.getImage();
	Collection properties = image.getProperties();
	Vector hibrowsefacets=new Vector();
	for (Iterator i=ui.getProfile().getStateOfHiBrowse().iterator(); i.hasNext();) {
	    FacetSelection fs=(FacetSelection)i.next();
	    MyFacetCombination mfs=new MyFacetCombination(fs.getActiveNode());
	    if (!hibrowsefacets.contains(mfs)) hibrowsefacets.add(mfs);
	}
	Vector imagefacets=new Vector();
	for (Iterator i=properties.iterator(); i.hasNext();) {
	    ResourceProperty rp=(ResourceProperty)i.next();
	    PropertyValue pv=rp.getValue();
	    if (pv.isInstance()) {
		MyFacetCombination mfs=new MyFacetCombination(((Instance)pv.getValue()).getType());
		if (!imagefacets.contains(mfs)) imagefacets.add(mfs);
	    }
	}
	for (Iterator i=hibrowsefacets.iterator(); i.hasNext();) {
	    MyFacetCombination mfs=(MyFacetCombination)i.next();
	    int indexOf=imagefacets.indexOf(mfs);
	    if (indexOf!=-1) {
		MyFacetCombination imfs=(MyFacetCombination)imagefacets.get(indexOf);
		while (!mfs.facetNode.equals(imfs.facetNode)) {
		    if (mfs.nodeDepth>imfs.nodeDepth) {
			mfs.nodeDepth--;
			Collection parents=mfs.facetNode.getParents(OntologyClass.DIRECT);
			Iterator itr=parents.iterator();
			if (itr.hasNext()) {
			    mfs.facetNode=(OntologyClass)itr.next();
			}
		    } else {
			imfs.nodeDepth--;
			Collection parents=imfs.facetNode.getParents(OntologyClass.DIRECT);
			Iterator itr=parents.iterator();
			if (itr.hasNext()) {
			    imfs.facetNode=(OntologyClass)itr.next();
			}
		    }
		    path++;
		}
	    } else {
		path+=mfs.nodeDepth;
	    }
	}
	int utility=MAXUTILITY/(path+1);
	log(il,utility);
	return utility;
    }
} 

