package fi.helsinki.w3m.filter;
import fi.helsinki.w3m.profile.Profile;
import java.util.Vector;
import java.util.Enumeration;

/*
 * $Author: jiemakel $
 * $Revision: 1.12 $
 * $Date: 2002/05/08 09:01:12 $
 */

/**
*This class joins the WeighedUtilityFunctions together in the chain. 
*Methods are given to manipulate the chain (add,remove,get) and to 
*calculate utilities of InterestingLinks, InterestingOntologyClasses and InterestingImages 
*by using the utilityfunctions in the chain.
*/



public class UtilityFunctionChain extends UtilityFunction {
    private Vector weighedUtilityFunctionChain;
    
    public UtilityFunctionChain() {
	weighedUtilityFunctionChain=new Vector();
    }
    
    public boolean addUtilityFunction(WeighedUtilityFunction wuf) {
	return weighedUtilityFunctionChain.add(wuf);
    }
    public boolean addUtilityFunction(UtilityFunction uf,int weight) {
	return weighedUtilityFunctionChain.add(new WeighedUtilityFunction(uf,weight));
    }
    
    public Vector getUtilityFunctionChain() {
	return weighedUtilityFunctionChain;
    }
    
    public boolean removeUtilityFunction(WeighedUtilityFunction uf) {
      return weighedUtilityFunctionChain.remove(uf);
    }
    
    public boolean removeUtilityFunction(int index) {
	if (weighedUtilityFunctionChain.remove(index)!=null) return true;
	else return false;
    }

/**
*Calculates the utility of an InterestingLink by summing (weighted) utilityfunction (the ones that are in the 
*chain) values of a given InterestingLink multiplied by the weights of the utilityfunctions and 
*dividing the sum by the sum 
*of weights of the utilityfunctions 
*
*@param il  The InterestingLink, whose utility is to be calculated
*@param prof The Profile of the caller
*@returns the integer division that is mentioned above 
*/
    
    public int getUtility(InterestingLink il, UtilityInformation ui) {
	int sum=0;
	int weightsum=0;
	WeighedUtilityFunction wuf;
	for (Enumeration e = weighedUtilityFunctionChain.elements(); e.hasMoreElements();) {
	    wuf=(WeighedUtilityFunction)e.nextElement();
	    sum+=wuf.getWeight()*wuf.getUtilityFunction().getUtility(il,ui);
	    weightsum+=wuf.getWeight();
	}
	log(il,sum/weightsum);
	return sum/weightsum;
    }

/**
*
*Calculates the utility of an InterestingOntologyClass by summing (weighted) utilityfunction (those that 
*are in the chain) values of a given InterestingOntologyClass multiplied 
*by the weights of the utilityfunctions and dividing the sum by the sum of weights of the utilityfunctions 
*
*@param ioc The InterestingOntologyClass, whose utility is to be calculated
*@param prof The Profile of the caller
*@returns the integer division mentioned above
*/

    public int getUtility(InterestingOntologyClass ioc, UtilityInformation ui) {
	int sum=0;
	int weightsum=0;
	WeighedUtilityFunction wuf;
	for (Enumeration e = weighedUtilityFunctionChain.elements(); e.hasMoreElements();) {
	    wuf=(WeighedUtilityFunction)e.nextElement();
	    sum+=wuf.getWeight()*wuf.getUtilityFunction().getUtility(ioc,ui);
	    weightsum+=wuf.getWeight();
	}
	log(ioc,sum/weightsum);
	return sum/weightsum;
    }
/**
*
*Calculates the utility of an InterestingImage by summing (weighted) utilityfunction (those that are in the 
*chain) values of a given InterestingImage multiplied by the 
*weights of the utilityfunctions and dividing the sum by the sum of weights of utilityfunctions 
*
*@param ii The InterestingImage, whose utility is to be calculated.
*@param prof The Profile of the caller
*@returns an integer division mentioned above
*/

    public int getUtility(InterestingImage ii, UtilityInformation ui) {
	int sum=0;
	int weightsum=0;
	WeighedUtilityFunction wuf;
	for (Enumeration e = weighedUtilityFunctionChain.elements(); e.hasMoreElements();) {
	    wuf=(WeighedUtilityFunction)e.nextElement();
	    sum+=wuf.getWeight()*wuf.getUtilityFunction().getUtility(ii,ui);
	    weightsum+=wuf.getWeight();
	}
	log(ii,sum/weightsum);
	return sum/weightsum;
    }

}
