package fi.helsinki.w3m.metadata;

import java.util.*;

/*
 * $Revision: 1.13 $
 * $Date: 2002/05/17 12:31:19 $
 */

/**
*An implementation of Id for use where a SimpleId will not be uniqe. Mostly for ResourceProperties.
*/
public class CompositeId implements Id {
	/**
	*The actual compostition if Id-objects.
	*/
    protected Collection ids;

    public final String ID_SEPARATOR = ";";
    
	/**
	*A constructor for the class.
	*
	*@param ids A CompositeId-object to clone.
	*/
    public CompositeId(Collection ids) {
	this.ids = ids;
    }

	/**
	*A constructor for the class.
	*
	*@param id1 An Id-object to add to the composition.
	*@param id2 An Id-object to add to the composition.
	*/
    public CompositeId(Id id1, Id id2) {
	this.ids = new Vector(2);
	this.ids.add(id1);
	this.ids.add(id2);
    }
	
	/**
	*Returns the id as a String value.
	*
	*@return The id as a String value.
	*/
    public String getId() {
	try{
		String rv = null;
		Iterator iter = ids.iterator();

		while (iter.hasNext()) {
	    	String idString = ((Id)iter.next()).getId();

	    	if (rv == null) rv = idString;
	    	else rv += ID_SEPARATOR + idString;
		}
	return rv;
	}
	catch(Exception e){
	    return null;
	}
    }	
	
	/**
	*Returns the id at position 'number'.
	*
	*@param number The position of the id to return.
	*@return The specified Id.
	*/
	public Id getId(int number) {
		try {
			return (Id)((Vector)ids).elementAt(number);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
    }	

	/**
	*Returns the id as a String value.
	*
	*@return The id as a String value.
	*/	
    public String getString() {
	return getId();
    }

	/**
	*Implements the Comparable interface.
	*/
    public int compareTo(Object anotherId) throws ClassCastException {
	int rv = 0;
	CompositeId another = (CompositeId)anotherId;
	Iterator thisIdIterator = this.ids.iterator();
	Iterator anotherIdIterator = another.ids.iterator();

	while ( thisIdIterator.hasNext() && anotherIdIterator.hasNext() && rv == 0 ) {
	    rv = ((Id)thisIdIterator.next()).compareTo(anotherIdIterator.next());
	}
	
	if ( rv == 0 ) {
	    if ( !thisIdIterator.hasNext() && anotherIdIterator.hasNext() ) {
		rv = -1;
	    }
	    else if ( thisIdIterator.hasNext() && !anotherIdIterator.hasNext() ) {
		rv = 1;
	    }
	}
	return rv;
    }

	/**
	*Returns the id as a String value.
	*
	*@return The id as a String value.
	*/
    public String toString() { 
		return getId(); 
	}
}
