package fi.helsinki.w3m.metadata;

import java.util.*;
import java.net.*;

import com.hp.hpl.mesa.rdf.jena.model.* ;
import com.hp.hpl.mesa.rdf.jena.common.* ;
import com.hp.hpl.jena.util.* ;

import fi.helsinki.w3m.metadata.ontology.*;
import fi.helsinki.w3m.metadata.item.*;

/*
 * $Revision: 1.18 $
 * $Date: 2002/05/17 12:31:19 $
 */

/**
 *An implementation of AbstractCollection to store OntologyResources.
 */
public abstract class ResourceCollection extends AbstractCollection {

    /**
     *Another ResourceCollection- object. Allows ResourceCollections to form a linked structure.
     */
    private ResourceCollection tail;

    /**
     *A constructor for this class.
     */ 
    protected ResourceCollection() {}

    /**
     *Returns a collection holding all OntologyResources in this ResourceCollection
     *by calling getList().
     *
     *@return A collection holding all OntologyResources in this ResourceCollection.
     */	
    public Collection getCollection() {
	return getList();
    }

    /**
     *Returns a linked list holding all OntologyResources in this ResourceCollection.
     *
     *@return A linked list holding all OntologyResources in this ResourceCollection.
     */
    public LinkedList getList() {
	LinkedList list = new LinkedList();
	Iterator iter = iterator();
	while (iter.hasNext()) {
	    list.add(iter.next());
	}
	return list;
    }

    /**
     *Adds the contents of another ResourceCollection to this ResourceCollection.
     *
     *@param resourceCollection Another ResourceCollection to add to this ResourceCollection. 
     *@return True if the operation succeeded.
     */
    public boolean addAll(Collection resourceCollection) {
	boolean rv = false;
	if (resourceCollection instanceof ResourceCollection) {
	    if (this.tail == null) {
		this.tail = (ResourceCollection)resourceCollection;
	    }
	    else {
		this.tail.addAll(resourceCollection);
	    }
	    rv = true;
	}
	return rv;
    }

    /**
     *Returns a new ResourceIterator for this ResourceCollection.
     *
     *@return new ResourceIterator for this ResourceCollection or null if this ResourceCollection is empty.
     */
    protected abstract ResourceIterator getIterator();


    /**
     * Constructs an iterator for this ResourceCollection. If tail exists
     * links it with this' iterator. 
     */
    public Iterator iterator() {
	ResourceIterator iterator = getIterator();

	// Create new empty ResourceIterator
	if (iterator == null) {
	    iterator = new ResourceIterator() {
		    protected Object getNext() { return null; }
		    protected boolean hasNextHead() { return false; }
		    protected void closeHead() {}
		};
	}

	// If there's tail, include it
	if (this.tail != null) {
	    iterator.addIterator((ResourceIterator)this.tail.iterator());
	}
	return iterator;
    }

    /**
     * Returns the number of elements in this ResourceCollection.
     *
     *@return An int value specifying the number of elements in this ResourceCollection.
     */
    public int size() {
	return getList().size();
    }
}
