package fi.helsinki.w3m.metadata;

import java.net.*;
import java.util.*;
import fi.helsinki.w3m.metadata.ontology.*;
import fi.helsinki.w3m.metadata.item.*;

import com.hp.hpl.mesa.rdf.jena.model.* ;
import com.hp.hpl.mesa.rdf.jena.common.* ;
import com.hp.hpl.jena.util.* ;

/*
 * $Revision: 1.51 $
 * $Date: 2002/05/17 12:31:19 $
 */
 
public class TestMuseumModel2 {

public static void main (String [] args) {
	boolean debug = false;
	int counter;
	if (args.length != 0) {
		debug = true;
	}
	try {
	    System.out.println("Creating model.");
	    final MuseumModel model = new MuseumModel(new URL("http://localhost/"), "../../../../../data");		
		model.readModel();
		System.out.println("4 most popular facets:");		
		//Iterator iter1 = model.getFacetRoots().iterator();		
		Iterator iter1 = model.getPopularFacets(4).iterator();		
		counter = 0;
		while (iter1.hasNext()) {	
			OntologyClass oclass = (OntologyClass)iter1.next();
			if (oclass != null)  {				
				counter ++;
				//oclass.addWeight();
				if (debug) System.out.println("   "+oclass.getId().toString()+ ", weight :"+ oclass.getWeight());
			}
		}
		System.out.println("   "+counter+" found.");		
		SimpleId oclassId = new SimpleId(model.encodeURI(model.BASE_NS+"Toimihenkil"));
		OntologyClass pclass = new OntologyClass(model, oclassId);

		System.out.println("Children for "+pclass.getLabel()+" :");		
		Iterator iter2 = pclass.getChildren(OntologyClass.ALL).iterator();
		counter = 0;		
		while (iter2.hasNext()) {
			OntologyClass oclass = (OntologyClass)iter2.next();
			counter ++;
			if (debug) System.out.println("   "+oclass.getId().toString());
		}
		System.out.println("   "+counter+" found.");
		System.out.println("Parents for "+pclass.getLabel()+" :");		
		Iterator iter3 = pclass.getParents(OntologyClass.ALL).iterator();
		counter = 0;
		while (iter3.hasNext()) {
			OntologyClass oclass = (OntologyClass)iter3.next();
			counter ++;
			if (debug) System.out.println("   "+oclass.getId().toString());
		}
		System.out.println("   "+counter+" found.");
		System.out.println("Instances for "+pclass.getLabel()+" :");
		Iterator iter4 = pclass.getInstances(OntologyClass.ALL).iterator();
		counter = 0;
		Instance pinst = (Instance)iter4.next();			
		while (iter4.hasNext()) {
			Instance instance = (Instance)iter4.next();			
			counter ++;
			model.setClassWeightMaximum(instance.getType().addWeight());
			if (debug) System.out.println("   "+instance.getId().toString()+"  class weight : "+ instance.getType().getWeight());
		}
		System.out.println("   "+counter+" found.");

		System.out.println("Images for "+pclass.getLabel()+" : (should be "+pclass.getImageCount()+")");
		Iterator iter5 = pclass.getImages(OntologyClass.ALL).iterator();
		counter = 0;		
		while (iter5.hasNext()) {
			Image image = (Image)iter5.next();			
			counter ++;
			model.setImageWeightMaximum(image.addWeight());
			if (debug) System.out.print("   "+image.getLabel()+"  weight : "+ image.getWeight()+"  URL : ");
			ImageFile ifile = image.getImageFile(ImageFile.ORIGINAL);
			if (debug) System.out.print(ifile.getBaseURL() + ifile.getFileName());

			/*
			Iterator ifiter = image.getImageFile(ImageFile.ORIGINAL).iterator();
			if (ifiter.hasNext()) {
				ImageFile ifile = (ImageFile)ifiter.next();
				if (debug) System.out.print(ifile.getBaseURL() + ifile.getFileName());
			}
			*/

			Iterator iter9 = image.getInstancesForClass(pclass).iterator();			
			while(iter9.hasNext()) {
				Instance inst9 = (Instance)iter9.next();				
				if (debug) System.out.print("  Instance by "+pclass.getLabel()+" : "+inst9.getLabel());
			}			
			if (debug) System.out.println();	
		}
		System.out.println("   "+counter+" found.");
		if (pinst != null) {
			System.out.println("Images for "+pinst.getLabel()+" : (should be "+pinst.getImageCount()+")");
			Iterator iter6 = pinst.getImages().iterator();
			counter = 0;		
			while (iter6.hasNext()) {
				Image image = (Image)iter6.next();			
				counter ++;
				if (debug) System.out.println("   "+image.getId().toString()+" Description: "+image.getDescription());
				
			}
			System.out.println("   "+counter+" found.");
			
			System.out.println("Properties for instance : "+pinst.getLabel()+" :");
			Iterator iter7 = pinst.getProperties().iterator();
			counter = 0;		
			while (iter7.hasNext()) {
				ResourceProperty prop = (ResourceProperty)iter7.next();			
				counter ++;
				if (debug) System.out.println("   "+prop.getLabel()+", value: "+(String)prop.getValue().getValue());
			}			
			System.out.println("   "+counter+" found.");
		}	

		Image testimage99 = new Image(model, new SimpleId(model.encodeURI("file:"+model.getBasePath()+"rdf/img_100296")));
		System.out.println("Description for image : "+testimage99.getDescription());		
		System.out.println("Properties for image : "+testimage99.getLabel()+" :");
		Iterator iter8 = testimage99.getProperties().iterator();
		counter = 0;		
		while (iter8.hasNext()) {
			ResourceProperty prop = (ResourceProperty)iter8.next();	
			if(prop.getValue().isInstance()) {
				if (debug) System.out.print("Instance");
				Instance newInst = prop.getValue().getInstance();
				if (debug) System.out.println(" loaded, label:" + newInst.getLabel());				
				if (debug) System.out.println("   "+prop.getLabel()+", value: "+prop.getValue().getInstance().getLabel());
			}
			counter ++;			
		}			
		System.out.println("   "+counter+" found.");
		
		counter = 0;
		Iterator allImageIter = model.getAllImages().iterator();
		while (allImageIter.hasNext()) {
			counter++;
			allImageIter.next();
		}					
		System.out.println("Total number of Images : " + counter);
		System.out.println("Image weight maximum : " + model.getImageWeightMaximum());
		System.out.println("Class weight maximum : " + model.getClassWeightMaximum());
		System.out.println("ImageProperty weight maximum : " + model.getImagePropertyWeightMaximum());
		//System.out.println("\n----------------\nWriting model.");				
		//model.writeModel();
		try {
			model.finalize();		
		} catch (Throwable t) {
			System.out.println("DIE!");				
		}
	} catch (Exception e) {
		e.printStackTrace();
	}
}

}
