package fi.helsinki.w3m.metadata.ontology;

import com.hp.hpl.mesa.rdf.jena.model.* ;
import com.hp.hpl.mesa.rdf.jena.common.* ;
import com.hp.hpl.jena.util.* ;

import fi.helsinki.w3m.metadata.*;
import fi.helsinki.w3m.metadata.ontology.*;
import fi.helsinki.w3m.metadata.item.*;

import java.util.*;

/*
 * $Revision: 1.28 $
 * $Date: 2002/05/17 12:31:22 $
 */

/**
 *An Instance of an OntologyClass
 */
public class Instance extends OntologyElement {

  /** TODO: An instance should be able to point to multiple ontology classes in some way.
	* This could be done by only allowing instance properties that point to ohter instances
	* and a mantadory property that points to a sort of "master instance".
	*/
    protected OntologyClass type;
	
   /**
    *This is used to avoid the need to recount the image count for an instance.
	*/
	private static Hashtable imageCountHash;

	static {
		/* Create the hashtable once for this class. */
		imageCountHash = new Hashtable();
	}
	
    public Instance(MuseumModel model, Id id) {
	super(model, id);
	this.getType();
    }
    
    public Instance(MuseumModel model, OntologyClass type, Id id) {
	super(model, id);	
	this.type = type;
    }

    /**
	 *Returns the OntologyClass-object(s) this object is an instance of.
     *
     *@returns The OntologyClass-object this object is an instance of. (Should return a collection ??)
     */
    public OntologyClass getType() {
	if (this.type == null) {
		try {
			ResourceImpl res = new ResourceImpl(this.getId().toString());		
			PropertyImpl pro = new PropertyImpl(model.TYPE_PROPERTY_URI);
			SelectorImpl selector = new SelectorImpl(res, pro, (String)null);
			final OntologyResource PARENT = this;
			ResourceProvider rp = new ResourceProvider() {
				public OntologyResource getResource( Object o ) {
					Statement stmt = (Statement) o;
					String subject = stmt.getObject().toString();			
				    return new OntologyClass(PARENT.getModel(), new SimpleId(subject));
				}
    		};	
			ResourceCollection rescoll = new StatementResourceCollection(model.getBaseModel(), selector, rp);
			Iterator resiter = rescoll.iterator();
			if (resiter.hasNext()) {
				OntologyClass oclass = (OntologyClass) resiter.next();
				this.type = oclass;
			}			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}	
	return this.type;
    }
	
    /**
     *Returns the images linked to this instance.
     *
     *@return A ResourceCollection-object holding the Image-objects for the images linked to this instance.
     */
	public ResourceCollection getImages() {
		StatementResourceCollection rv;
		try {
			PropertyImpl pro = new PropertyImpl(type.getId().toString());
		    SelectorImpl selector = new SelectorImpl(null, pro, this.getId().toString());
			ResourceProvider rp = new ResourceProvider() {
				public OntologyResource getResource( Object o ) {
					Statement stmt = (Statement) o;
					String subject = stmt.getSubject().toString();				
				    return new Image(model, new SimpleId(subject));
				}
	    	};	
			rv = new StatementResourceCollection(model.getBaseModel(), selector, rp);
		    return rv;
		} catch (RDFException rdfe) {
			rdfe.printStackTrace();
			return null;
		}
	}
	
   /**
     * Returns the number of images linked to an instance.
	 *
	 *@return A long value holding the number of images linked to this ontology class.
     */
    public long getImageCount() {
		if (imageCountHash.containsKey(this.getId())) {			
			return ((Long)imageCountHash.get(this.getId())).longValue();
		} 
		try {
			long rv = 0;	
			PropertyImpl pro = new PropertyImpl(type.getId().toString());
		    SelectorImpl selector = new SelectorImpl(null, pro, this.getId().toString());
			/* No need to use the ResourceCollection-infrastructure as we just count the statements. */
			StmtIterator stmtIter = model.getBaseModel().listStatements(selector);
			while (stmtIter.hasNext()) {
				rv++;
				stmtIter.next();
			}
			/* Save the count in the count hash */			
			imageCountHash.put(this.getId(), new Long(rv));
			return rv;					
		} catch (RDFException rdfe) {
			rdfe.printStackTrace();
			return -1;
		}		
    }

    /**
     *Gives the label of this instance.
     *
     *@return value of the MuseumModel.INSTANCELABEL_PREDICATE-property of this instance.
     */
    public String getLabel() {
		try {
			return (String)this.getNamedProperty(model.INSTANCELABEL_PREDICATE, model.getBaseModel()).getValue().getLiteral();
		} catch (Exception e) {
			/* Return something at least. */
			return this.getType().getLabel();
		}
    }
}
