package fi.helsinki.w3m.metadata.ontology;

import java.util.*;
import java.net.*;

import fi.helsinki.w3m.metadata.*;
import fi.helsinki.w3m.metadata.item.*;

/*
 * $Revision: 1.27 $
 * $Date: 2002/05/17 12:31:23 $
 */

/**
 *A wrapper class for the value of a property. Needed since the value can be an OntologyResource.
 */
public class PropertyValue extends OntologyResource {
    public final static Class LITERAL = String.class;
    public final static Class INSTANCE = Instance.class;
    public final static Class INTEGER = Integer.class;

    protected Object value;
    protected Class type;
    protected boolean isBag;

    /**
     *Constructs a new Property ant tries to figure out the value's type. 
     */
    public PropertyValue(Object value, MuseumModel model) throws ClassNotFoundException {
	super(model, new SimpleId(value.toString()));

	this.value = value;
	Class vtype = this.value.getClass();
	
	if (vtype.isAssignableFrom(java.util.Collection.class )) {
	    this.isBag = true;

	    /* Try to figure out this Bags component type. */
	    Iterator i = ((Collection)this.value).iterator();
	    if ( i.hasNext() ) {
		vtype = i.next().getClass();
	    }

	    /* If collection is empty, then set type = Object - most generic. */
	    else {
		vtype = Object.class;
	    }
	}

	this.type = vtype;
    }


    /**
     *@return The value this PropertyValue wraps or null if its type
     *is not Instace.
     */
    public Instance getInstance() throws IllegalStateException {
	Instance rv = null;
	if (isInstance()) {
	    rv = (Instance)this.value;
	}
	return rv;
    }

    public int getInt() throws IllegalStateException {
	if (isInt()) {
	    return ((Integer)this.value).intValue();
	}
	else {
	    throw new IllegalStateException("No int value!");
	}
    }

    /**
     *@return The value this PropertyValue wraps or null if its type
     *is not RDF literal.
     */
    public String getLiteral() {
	String rv = null;
	if (isLiteral()) {
	    rv = (String)this.value;
	}
	return rv;
    }

    /**
     *Returns the type of the value this PropertyValue wraps.
     *
     *@return A Class-object holding the class the object this PropertyValue wraps is an instance of.
     */
    public Class getType() {
	return this.type;
    }
	
    /**
     *Overrides the getWeight()- method in OntologyResource since it is not sane to give weights to PropertyDefinition-objects. Give them to ResourceProperties instead. Calling this method will throw an exception.
     *
	 *@throws UnsupportedOperationException It is not sane to give weights to instances of this class.
	 *@see OntologyResource#getWeight()
     *@return Nothing. (But supposedly a long value)
     */
    public long getWeight(){
	throw new UnsupportedOperationException("This class has no weight.");
    }

    /**
     *Returns the value this PropertyValue wraps.
     *
     *@return An Object holding the value this PropertyValue wraps.
     */
    public Object getValue() { return this.value; }

    /**
     *@return True if the value this PropertyValue wraps implements Comparable.
     */	
    public boolean isComparable() {
	return this.type.isAssignableFrom( java.lang.Comparable.class );
    }

    /**
     *@return True if the value this PropertyValue wraps is an RDF bag.
     */
    public boolean isBag() { return this.isBag; }

    /**
     *@return True if the value this PropertyValue wraps is an Instance.
     */
    public boolean isInstance() {
	return this.value instanceof Instance;
    }

    public boolean isInt() {
	boolean isInt = false;
	if (this.value instanceof Integer) {
	    isInt = true;
	}
	else if (isLiteral()) {
	    try {
		this.value = new Integer(getLiteral());
		this.type = INTEGER;
		isInt = true;
	    } catch (NumberFormatException nfe) {}
	}
	return isInt;
    }

    /**
     *@return True if the value this PropertyValue wraps is an RDF literal.
     */
    public boolean isLiteral() {
	return this.value instanceof String;
    }
    /**
     *Overrides the addWeight()- method in OntologyResource since it is not sane to give weights to PropertyDefinition-objects. Give them to ResourceProperties instead. Calling this method will throw an exception
     *
 	 *@throws UnsupportedOperationException It is not sane to give weights to instances of this class.
	 *@see OntologyResource#addWeight()
     *@return Nothing. (But supposedly a long value)
     */	
    public long addWeight(){
	throw new UnsupportedOperationException("This class has no weight.");
    }   
}
