package fi.helsinki.w3m.metadata.ontology;

import java.util.*;
import java.net.*;

import fi.helsinki.w3m.metadata.*;
import fi.helsinki.w3m.metadata.item.*;

/*
 * $Revision: 1.12 $
 * $Date: 2002/05/17 12:31:23 $
 */

/**
*A property of an OntologyResource.
*/
public class ResourceProperty extends PropertyDefinition {
    protected Object value;
    private String label;

    /**
     *Constructs a new ResourceProperty.
	 *
	 *@param property The definition of this property.
	 *@param resource The OntologyResource to which this ResourceProperty belongs.
	 *@param value The value of this ResourceProperty.
	 *@param model The MuseumModel this ResourceProperty belongs to.
     */
    public ResourceProperty(PropertyDefinition property, OntologyResource resource, Object value, MuseumModel model) {
	super(model, new CompositeId(property.getId(), resource.getId()));
	this.value = value;
	
	if (this.id instanceof CompositeId) {
		this.label = ((CompositeId)this.getId()).getId(0).toString();
		this.label = this.label.substring(this.label.lastIndexOf(model.BASE_NS) + model.BASE_NS.length());
		this.label = MuseumModel.decodeURI(this.label);
	}
    }

	public String getLabel() { return this.label; }
	
	/**
	*Returns the value of this ResourceProperty
	*
	*@return A PropertyValue-object holding the value of this property.
	*/
    public PropertyValue getValue() {
		try {
			return new PropertyValue(this.value, this.model);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
    }
	
}
