/**
 * University of Helsinki
 * WWW-Museum
 *
 * @author Pasi Lehtimki, Samppa Saarela
 *
 * FacetSelection is a wrapper class for one selected facet.
 */

package fi.helsinki.w3m.profile;

import java.util.Vector;

import fi.helsinki.w3m.metadata.ontology.OntologyClass;

public class FacetSelection implements Comparable{

    //Selected class
    private OntologyClass activeNode = null;

    //root class of this facet
    private OntologyClass facetRoot = null;

    //path from root to selected class. Class ids separated with Profile.ONTOLOGY_CLASS_SEPARATOR
    private String selectionPath = "";
    
    //The way this facet's images should be ordered. 
    private int orderBy;

    //individual id of this facet
    private int id;

    //list of classes that user has set open. (Strings of class ids)
    private Vector openedNodes;


    /**
     * Constructs a new FacetSelection
     */

    FacetSelection( int id ) {
	    
	this.orderBy = Profile.ORDER_BY_IMAGES;
	this.id = id;
    }


    /**
     * Constructs a new FacetSelection
     */
    public FacetSelection( int id, OntologyClass facetRootNode) {
	this.id = id;
	this.facetRoot = facetRootNode;
	//this.activeNode = facetRootNode;
	this.openedNodes = new Vector();
	addOpen(this.facetRoot.getId().getString());

    }


    /**
     * @return int ordering info
     */
    public int getOrderBy(){
	  return this.orderBy;
    }

    /**
     * @param i ordering info
     */
    
    public void setOrderBy(int i){
	    System.out.println("fs order by: "+i);
	    this.orderBy = i;
    }
    

    public int compareTo( Object another ) {
	FacetSelection fs = (FacetSelection)another;
	if ( this.id < fs.id ) return -1;
	else if ( this.id == fs.id ) return 0;
	else return 1;
    }

    public boolean equals( Object another ) {
	return compareTo(another) == 0;
    }

	
    /**
     * @return selected OntologyClass
     */
    public OntologyClass getActiveNode(){
	return this.activeNode;
    }

    /**
     * @return the root OntologyClass of this facet
     */	
    public OntologyClass getFacetRoot(){
	return this.facetRoot;
    }

    /**
     * @return individual id of this facet
     */

    public int getId() {
	return this.id;
    }
	

    /**
     * @return name of this facet
     */
    public String getName() {
	return this.facetRoot.getLabel();
    }


    /**
     * Sets this facets selection path
     */
    public void setSelectionPath(String path) {
	this.selectionPath = path;
    }
	

    /**
     * @return String selection path from root to selected class. Separated by Profile.ONTOLOGY_CLASS_SEPARATOR
     */
    public String getSelectionPath(){
	return this.selectionPath;
    }
	

    /**
     * Sets the selected OntologyClass of this facet
     */
    public void setActiveNode(OntologyClass node){
	this.activeNode = node;
    }

    /**
     * Adds one class to the list of opened classes
     */
    public void addOpen(String name){
	this.openedNodes.add(name);
    }


    /**
     * Removes the class from the list of opened classes
     */
    public void removeOpened(String name) {
	this.openedNodes.remove(name);
    }
	

    /**
     * Returns true if "name" can be found from the list of opened classes
     */
    public boolean isOpen(String name){
	return this.openedNodes.contains(name);
    }
}

