package fi.helsinki.w3m.ui.servlet;

import javax.servlet.http.*;

import fi.helsinki.w3m.profile.*;

import javax.servlet.*;
import javax.servlet.http.*;

import java.lang.reflect.*;
import java.util.*;


public class ActionServlet extends HttpServlet{

    /** The name of the http-parameter that defines action. (action) */
    public static String ACTION  = "action";

    /** The name of the default target jsp.
	This is used if target parameter is omitted.*/
    public static String DEFAULT_TARGET = "default";

    /** The name of the http-parameter that tell's if debugging information 
	should be printed to stdout. (debug)  */
    public static String DEBUG = "debug";

    /** The name of the http-parameter that defines facet's name. (facet) */
    public static String FACET   = "facet";

    /** The name of the http-parameter that defines class' name. (class) */
    public static String CLASS   = "class";

    /** The name of the http-parameter that defines the names of the
	http-parameters that correspond to parameter's of Profiles method
	to be called. */
    public static String PARAM_NAMES = "paramNames";

    /** The name of the HttpSessio-parameter that stores user's profile. */
    public static String PROFILE = "profile";

    /** The name of the http-parameter that defines the target jsp-page
     to which the request should be redirected.*/
    public static String TARGET  = "target";

    /** URI's of the target jsp's.
	<p>
	targets.put( <target's name>, <target's uri> )
    */
    private static Hashtable targets = new Hashtable();

    static {
	// targets.put( <target's name>, <target's uri> )
	targets.put("main", "/wwwmuseo/result.jsp");
	targets.put(DEFAULT_TARGET, "/wwwmuseo/result.jsp");
	targets.put("hiBrowse", "/wwwmuseo/hiBrowse.jsp");
    }

    /**
       <p>
       This method first updates user's profile by calling the method 
       defined by http-parameter <tt>action</tt>. 
       Then the request will be redirected to handler which 
       is defined by <tt>target</tt>. Possible targets are 
       defined in targets.

       <p>
       Http-parameter's:
       <dl>
       <dt>action</dt>
       <dd>The name of the Profile's method that should be called with 
       parameter's listed in <tt>paramNames</tt>.</dd>
       <dt>debug</dt>
       <dd>If debugging information should be printed to stdout.</dd>
       <dt>paramNames</dt>
       <dd>List of the names of the http-parameters delimited with "," 
       which correspond to the parameters
       of Profile's methdod to be called. If this parameter is omitted
       http-parameter's <tt>facet</tt> and <tt>class</tt> will be used.</dd>
       <dt>target</dt>
       <dd>The name of the JSP-page to which the request should be redirected [main | HiBrowse].</dd>
       <dt>class</dt>
       <dd>The default name of the http-parameter that will be used as a second
       parameter for the method <tt>action</tt> if <tt>paramNames</tt>  is omitted.</dd>
       <dt>facet</dt>
       <dd>The default name of the http-parameter that will be used as a first
       parameter for the method <tt>action</tt> if <tt>paramNames</tt> is omitted.</dd>
       </dl>

       <h3>Example</h3>
       If one needs to update user's profile by calling Profile's 
       (HiBrowse related) method 
       <tt>closeFacet(String facet, String class)</tt> he should 
       provide the following http-parameters: 
       <ul>
       <li>action=closeFacet</li>
       <li>paramNames=facet,class</li>
       <li>class=&lt;the ID of the class to be closed&gt;</li>
       <li>facet=&lt;the ID of the facet into which the <tt>class</tt> belongs to&gt;</li>
       <li>target=hiBrowse</li>
       </ul>
     */
    public void service( HttpServletRequest req, HttpServletResponse res ) {
	HttpSession session = req.getSession();
	Profile profile = null;
	String redirParams = null;
	Object tmp = null;

	String action     = nullIfEmpty( req.getParameter( ACTION ) );
	boolean debug     = req.getParameter( DEBUG ) != null;
	String paramNames = req.getParameter( PARAM_NAMES );
	String target     = nullIfEmpty( req.getParameter( TARGET ) );

	// target-parameter is required
	if ( target == null || !targets.containsKey( target ) ) {
	    target = DEFAULT_TARGET;
	}

	// Get user's profile
	tmp = session.getAttribute( PROFILE );
	if ( tmp == null ) {
	    profile = new Profile();
	    session.setAttribute( PROFILE, profile );
	}
	else {
	    profile = (Profile)tmp;
	}
	
	// Update the profile
	if ( action != null ) {
	    try {
		Class[] ptypes = null;
		String[] params = null;

		// Used to track what method is called and with what parameters
		String tracker = action  + "(";

		// Use the parameter's given
		if ( paramNames != null ) {
		    StringTokenizer st = new StringTokenizer( paramNames, "," );
		    Vector ptypesVec = new Vector();
		    Vector paramsVec = new Vector();
		    String pname = null;

		    while ( st.hasMoreTokens() ) {
			pname = nullIfEmpty( st.nextToken() );
			if ( pname != null ) {
			    ptypesVec.add( String.class );
			    paramsVec.add( req.getParameter( pname ) );
			    
			    // Tracking...
			    if ( debug ) {
				if ( paramsVec.size() > 1 )
				    tracker += ", ";
				tracker += req.getParameter( pname );
			    }
			}
		    }

		    ptypes = new Class[ ptypesVec.size() ];
		    params = new String[ paramsVec.size() ];

		    ptypes = (Class[])ptypesVec.toArray( ptypes );
		    params = (String[])paramsVec.toArray( params );
		}

		// Use default parameter's 
		else {
		    ptypes = new Class[2];
		    params = new String[2];

		    ptypes[0] = String.class;
		    ptypes[1] = String.class;
		    params[0] = req.getParameter( FACET );
		    params[1] = req.getParameter( CLASS );

		    if ( debug ) {
			tracker += req.getParameter( FACET ) + ", " + req.getParameter( CLASS );
		    }
		}

		if ( debug ) {
		    System.out.println( "ActionServlet: Call Profile's method: "+tracker +")");
		}

		// Call the method with given parameters
		Method handler = Profile.class.getDeclaredMethod( action, ptypes );
		handler.invoke( profile, params );
	    } catch ( Exception e ) {
		e.printStackTrace( System.err );
	    }
	}

	// Redirect the request to jsp
	try {
	    redirParams = req.getQueryString();
	    if ( redirParams == null ) redirParams = "";
	    else redirParams = "?"+redirParams;

	    if ( debug ) {
		System.out.println( "ActionServlet: redirect: "+(String)targets.get(target)+redirParams );
	    }
	    res.sendRedirect( res.encodeURL( (String)targets.get(target)+redirParams ) );
	} catch ( Exception e ) {
	    e.printStackTrace( System.err );
	}
    }

    /**
       Returns null if str is null or str.trim() is empty String else srt.trim().
    */
    public static String nullIfEmpty( String str ) {
	String rv = null;
	if ( str != null ) {
	    str = str.trim();
	    if ( !str.equals("") ) 
		rv = str;
	}
	return rv;
    } 
}

